# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.astronomy.sun import AstronomySunRequest, AstronomySunResponse
from pyqweather.packages.astronomy.moon import AstronomyMoonRequest, AstronomyMoonResponse
from pyqweather.packages.astronomy.solar_elevation_angle import AstronomySolarElevationAngleRequest, AstronomySolarElevationAngleResponse



class QWeatherAstronomyPack(QWeatherPackBase):
  """天文API包"""
  
  def sun(self, location:str, date:str) -> AstronomySunResponse:
    """获取未来60天全球任意地点日出日落时间。
    
    Args:
        location (str): (必选)需要查询地区的LocationID或以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位），LocationID可通过GeoAPI获取。例如 location=101010100 或 location=116.41,39.92
        date (str): (必选)选择日期，最多可选择未来60天（包含今天）的数据。日期格式为yyyyMMdd，例如 date=20200531

    Returns:
        AstronomySunResponse: _description_
    """
    
    req = AstronomySunRequest(location=location,date=date)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return AstronomySunResponse(**data)
  
  
  def moon(self, location:str, date:str , lang:str='zh-hans') -> AstronomyMoonResponse:
    """月升月落和月相

    Args:
        location (_type_): (必选)需要查询地区的LocationID或以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位），LocationID可通过GeoAPI获取。例如 location=101010100 或 location=116.41,39.92
        date (str): (必选)选择日期，最多可选择未来60天（包含今天）的数据。日期格式为yyyyMMdd，例如 date=20200531
        lang (str, optional): 多语言设置. Defaults to 'zh-hans'.


    Returns:
        AstronomyMoonResponse: _description_
    """
    
    req = AstronomyMoonRequest(location=location, date=date,  lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return AstronomyMoonResponse(**data)
  
  
  def solar_elevation_angle(self, location:str, date:str, time:str, alt:str, tz:str='0800') -> AstronomySolarElevationAngleResponse:
    """任意时间点的全球太阳高度及方位角。

    Args:
        location (str): (必选)需要查询地区的以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位）。例如 location=116.41,39.92
        date (str): (必选)查询日期，格式为yyyyMMdd，例如 date=20170809
        time (str): (必选)查询时间，格式为HHmm，24时制，例如 time=1230
        alt (str): (必选)海拔高度，单位为米，例如alt=43
        tz (str, optional): 查询地区所在时区，例如tz=0800或tz=-0530 ，默认0800

    Returns:
        AstronomySolarElevationAngleRequest: _description_
    """
    req = AstronomySolarElevationAngleRequest(location=location, date=date, time=time,  alt=alt, tz=tz)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return AstronomySolarElevationAngleResponse(**data)
  
  
  



from pyqweather import QWeatherRequestBase, QWeatherResponseBase

class AstronomySunRequest(QWeatherRequestBase):
  
  _PATH = '/astronomy/sun'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.location = self.get_arg('location', kwargs, required=True)
    self.date = self.get_arg('date', kwargs, required=True)
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class AstronomySunResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.sunrise= self.get_arg('sunrise', kwargs, None)
    self.sunset= self.get_arg('sunset', kwargs, None)
    


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.indices.indices_daily import IndicesDailyRequest, IndicesDailyResponse


class QWeatherIndicesPack(QWeatherPackBase):
  """天气指数API包"""
  
  def indices_1d(self, location:str, type, lang:str='zh-hans'):
    """1日天气指数预报

    Args:
        location (str): (必选)需要查询地区的LocationID或以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位），LocationID可通过GeoAPI获取。例如 location=101010100 或 location=116.41,39.92
        type(str): (必选)生活指数的类型ID，包括洗车指数、穿衣指数、钓鱼指数等。可以一次性获取多个类型的生活指数，多个类型用英文,分割。例如type=3,5。具体生活指数的ID和等级参考天气指数信息。各项生活指数并非适用于所有城市。
        lang (str, optional):多语言设置，请阅读多语言文档，了解我们的多语言是如何工作、如何设置以及数据是否支持多语言。 Defaults to 'zh-hans'.

    Returns:
        _type_: _description_
    """    
    
    req = IndicesDailyRequest(day_range='1d', location=location, type=type, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return IndicesDailyResponse(**data)
  
  
  def indices_3d(self, location:str, type, lang:str='zh-hans'):
    """3日天气指数预报

    Args:
        location (str): (必选)需要查询地区的LocationID或以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位），LocationID可通过GeoAPI获取。例如 location=101010100 或 location=116.41,39.92
        type(str): (必选)生活指数的类型ID，包括洗车指数、穿衣指数、钓鱼指数等。可以一次性获取多个类型的生活指数，多个类型用英文,分割。例如type=3,5。具体生活指数的ID和等级参考天气指数信息。各项生活指数并非适用于所有城市。
        lang (str, optional):多语言设置，请阅读多语言文档，了解我们的多语言是如何工作、如何设置以及数据是否支持多语言。 Defaults to 'zh-hans'.

    Returns:
        _type_: _description_
    """    
    
    req = IndicesDailyRequest(day_range='3d', location=location, type=type, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return IndicesDailyResponse(**data)

    
  


# coding: utf-8

from pyqweather import QWeatherConfig
from pyqweather.packages.geo import QWeatherGeoPack
from pyqweather.packages.weather import QWeatherWeatherPack
from pyqweather.packages.minutely import QWeatherMinutelyPack
from pyqweather.packages.grid_weather import QWeatherGridWeatherPack
from pyqweather.packages.weather_warning import QWeatherWeatherWarningPack
from pyqweather.packages.indices import QWeatherIndicesPack
from pyqweather.packages.air_quality import QWeatherAirQualityPack
from pyqweather.packages.air import QWeatherAirPack
from pyqweather.packages.time_machine import QWeatherTimeMachinePack
from pyqweather.packages.tropical_cyclone import QWeatherTropicalCyclonePack
from pyqweather.packages.ocean import QWeatherOceanPack
from pyqweather.packages.solar_radiation import QWeatherSolarRadiationPack
from pyqweather.packages.astronomy import QWeatherAstronomyPack


class QWeatherFactory:
  
  def __init__(self):
    self._pack_dict = {
      'geo': { 'ns': 'pyqweather.requests.geo', 'module': 'QWeatherGeoPack' }
    }
  
  
  def create_pack(self, packName: str, config: QWeatherConfig):
    
    if packName not in self._pack_dict:
      return None
    
    packInfo = self._pack_dict[packName]
    ns = __import__(packInfo['ns'], fromlist=['None'])
    moduleName = packInfo['module']
    moduleInstance = eval(f'ns.{moduleName}(config)')
    return moduleInstance
  
  
  def create_geo_pack(self, config: QWeatherConfig) -> QWeatherGeoPack:
    """创建 Geo API包"""
    
    return QWeatherGeoPack(config)
  
  
  def create_weather_pack(self, config: QWeatherConfig) -> QWeatherWeatherPack:
    """创建城市天气预报API包"""
    
    return QWeatherWeatherPack(config)
  
  
  def create_minutely_pack(self, config: QWeatherConfig) -> QWeatherMinutelyPack:
    """创建分钟级降水API（临近预报）包
    """
    
    return QWeatherMinutelyPack(config)
  
  
  def create_grid_weather_pack(self, config: QWeatherConfig) -> QWeatherGridWeatherPack:
    """以经纬度为基准的全球高精度、公里级、格点化天气预报产品，包括任意经纬度的实时天气和天气预报。"""
    
    return QWeatherGridWeatherPack(config)
  
  
  def create_weather_warning_pack(self, config: QWeatherConfig) -> QWeatherWeatherWarningPack:
    """创建预警API包"""
    
    return QWeatherWeatherWarningPack(config)
  
  
  def create_indices_pack(self, config: QWeatherConfig) -> QWeatherIndicesPack:
    """创建中国和全球城市天气生活指数预报数据API包"""
    
    return QWeatherIndicesPack(config)
  
  
  def create_airquality_pack(self, config: QWeatherConfig) -> QWeatherAirQualityPack:
    """创建空气质量(beta)API包"""
    
    return QWeatherAirQualityPack(config)
  
  def create_air_pack(self, config: QWeatherConfig) -> QWeatherAirPack:
    """创建空气质量 API包"""
    
    return QWeatherAirPack(config)
  
  
  def create_time_machine_pack(self, config: QWeatherConfig) -> QWeatherTimeMachinePack:
    """创建时光机 API包"""
    
    return QWeatherTimeMachinePack(config)
  
  
  def create_tropical_cyclone_pack(self, config: QWeatherConfig) -> QWeatherTropicalCyclonePack:
    """创建热带气旋（台风） API包
    
    ```热带气旋（台风）API提供全球主要海洋流域的台风信息，包括台风实时位置、等级、气压、风速，还可查询台风路径和台风预报信息。```
    """
    
    return QWeatherTropicalCyclonePack(config)
  
  
  def create_ocean_pack(self, config: QWeatherConfig) -> QWeatherOceanPack:
    """创建海洋数据 API包
    
    ```海洋数据API提供全球主要港口和城市的潮汐和潮流数据。```
    """
    
    return QWeatherOceanPack(config)
  
  
  def create_solar_radiation_pack(self, config: QWeatherConfig) -> QWeatherSolarRadiationPack:
    """创建太阳辐射 API包
    
    ```太阳辐射API支持获取全球任意坐标的辐射数据，包括净太阳辐射，太阳散射辐射和太阳直接辐射。```
    """
    
    return QWeatherSolarRadiationPack(config)
  
  
  def create_astronomy_pack(self, config: QWeatherConfig) -> QWeatherAstronomyPack:
    """创建天文API包

    Args:
        config (QWeatherConfig): _description_

    Returns:
        QWeatherAstronomyPack: _description_
    """
    return QWeatherAstronomyPack(config)



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherTropicalCycloneStormDto


class StormListRequest(QWeatherRequestBase):
  
  _PATH = '/tropical/storm-list'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.basin = self.get_arg('basin', kwargs, required=True)
    self.year = self.get_arg('year', kwargs, required=True)
    
    
  def __str__(self) -> str:
    return f'stormid={self.stormid}'
  

class StormListResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.storm:list[QWeatherTropicalCycloneStormDto] = self.get_items('storm', kwargs, QWeatherTropicalCycloneStormDto)
    



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherAirQualityStationPollutantDto


class AirQualityStationRequest(QWeatherRequestBase):
  
  _PATH = '/station'
  
  def __init__(self, locationId:str,  **kwargs):
    super().__init__(**kwargs)
    self.locationId = locationId
    self.lang = self.get_arg('lang', kwargs)
    
    self._PATH = f'/station/{self.locationId}'
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  


class AirQualityStationResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.pollutant:list[QWeatherAirQualityStationPollutantDto] = self.get_items('pollutant', kwargs, QWeatherAirQualityStationPollutantDto)
    


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.minutely.minutely_5m import Minutely5mRequest, Minutely5mResponse


class QWeatherMinutelyPack(QWeatherPackBase):
  """分钟级降水API（临近预报）"""
  
  def minutely_5m(self, location, lang:str='zh-hans'):
    """中国1公里精度的未来2小时每5分钟降雨预报数据。"""
    
    req = Minutely5mRequest(location=location,lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return Minutely5mResponse(**data)
  


# coding : utf-8
from abc import abstractmethod, ABC
from pyqweather.auth import AuthCredentialBase
import requests
from requests import Response
from json import dumps
from pyqweather.packages import QWeatherReferDto

class QWeatherConfig:
  """配置类
  """
  
  def __init__(self, endpoint:str, credential:AuthCredentialBase=None):
    self._credential = credential
    self.endpoint = endpoint


  def get_credential(self):
    return self._credential


class QWeatherPackBase(ABC):
  """QWeather接口包基类

  """
    
  def __init__(self, conf:QWeatherConfig):
    self._conf = conf
    
  
  def get_conf(self):
    return self._conf
  
  
class QWeatherRequestBase(ABC):
  """QWeather请求对象
  """
  
  
  _PATH = ''
  
  
  def __init__(self, **kwargs):
    self._kwargs = kwargs
  
  
  def with_url(self, url:str):
    self._url = url
    return self
    
  
  def with_credential(self, credential:AuthCredentialBase=None):
    self._credential = credential
    return self
    
    
  def get_arg(self, name, dict, default=None, required=False):
    arg = dict[name] if name in dict else default
    if required and dict[name] is None :
      raise Exception(f'{name} is required ,but missing now.')
    
    return arg
    
    
  def get(self) -> dict:
    if not self._credential.is_valid():
      raise Exception(f'Auth credential required, but it is empty or None') 
    
    fullurl = f'{self._url}{self._PATH}'
    print(fullurl)
    self._kwargs.update(self._credential.create_sign_param(**self._kwargs))
    resp = requests.get(fullurl, params=self._kwargs)
    
    print(resp.url)
    if resp.status_code != 200:
      raise Exception(f'response status code {resp.status_code}, at {fullurl}')
    
    return resp.json()
    
  
  
class QWeatherResponseBase(ABC):
  
  def __init__(self, **kwargs):
    self.code = self.get_arg('code', kwargs)
    """状态码"""
    self.refer:QWeatherReferDto = self.get_obj('refer', kwargs, QWeatherReferDto)
    """原始数据引用信息"""
    
  def __str__(self) -> str:
    return dumps(self.__dict__)
    
  def get_arg(self, name, dict, default=None):
    return dict[name] if name in dict else default
    
    
  def get_obj(self, name, dict, cls:any):
    return cls(**dict[name]) if name in dict else None
    
    
  def get_items(self, name, dict, cls:any):
    return list(map(lambda x: cls(**x), dict[name])) if name in dict else []
    
    
  def get_code(self) -> str:
    return self.code
  
  

  


#coding : utf-8

from enum import Enum
from abc import ABC

class QWeatherDto(ABC):
  
  def __init__(self, **data):
    self.__dict__.update(data)


class QWeatherReferDto(QWeatherDto):
  sources : str = None
  """原始数据来源，或数据源说明，可能为空"""
  license : str = None
  """数据许可或版权声明，可能为空"""


class QWeatherWeatherDataDto(QWeatherDto):
  """城市天气预报数据"""
  
  obsTime : str = None
  """数据观测时间"""
  temp  : str = None
  """温度，默认单位：摄氏度"""
  feelsLike  : str = None
  """体感温度，默认单位：摄氏度"""
  icon   : str = None
  """天气状况的图标代码"""
  text  : str = None
  """天气状况的文字描述，包括阴晴雨雪等天气状态的描述"""
  wind360  : str = None
  """风向360角度"""
  windDir  : str = None
  """风向"""
  windScale  : str = None
  """风力等级"""
  windSpeed  : str = None
  """风速，公里/小时"""
  humidity  : str = None
  """相对湿度，百分比数值"""
  precip  : str = None
  """过去1小时降水量，默认单位：毫米"""
  pressure  : str = None
  """大气压强，默认单位：百帕"""
  vis  : str = None
  """能见度，默认单位：公里"""
  cloud  : str = None
  """云量，百分比数值。可能为空"""
  dew  : str = None
  """露点温度。可能为空"""
  
  
class QWeatherWeatherDailyDataDto(QWeatherDto):
  """每日天气预报数据"""
  
  fxDate: str = None
  """预报日期"""
  sunrise : str = None
  """ 日出时间，在高纬度地区可能为空"""
  sunset : str = None
  """ 日落时间，在高纬度地区可能为空"""
  moonrise : str = None
  """当天月升时间，可能为空"""
  moonset : str = None
  """当天月落时间，可能为空"""
  moonPhase : str = None
  """月相名称"""
  moonPhaseIcon : str = None
  """月相图标代码"""
  tempMax : str = None
  """预报当天最高温度"""
  tempMin : str = None
  """预报当天最低温度"""
  iconDay : str = None
  """预报白天天气状况的图标代码"""
  textDay : str = None
  """预报白天天气状况文字描述，包括阴晴雨雪等天气状态的描述"""
  iconNight : str = None
  """预报夜间天气状况的图标代码，"""
  textNight : str = None
  """预报晚间天气状况文字描述，包括阴晴雨雪等天气状态的描述"""
  wind360Day : str = None
  """预报白天风向360角度"""
  windDirDay : str = None
  """预报白天风向"""
  windScaleDay : str = None
  """预报白天风力等级"""
  windSpeedDay : str = None
  """预报白天风速，公里/小时"""
  wind360Night : str = None
  """预报夜间风向360角度"""
  windDirNight : str = None
  """预报夜间当天风向"""
  windScaleNight : str = None
  """预报夜间风力等级"""
  windSpeedNight : str = None
  """预报夜间风速，公里/小时"""
  precip : str = None
  """预报当天总降水量，默认单位：毫米"""
  uvIndex : str = None
  """紫外线强度指数"""
  humidity : str = None
  """相对湿度，百分比数值"""
  pressure : str = None
  """大气压强，默认单位：百帕"""
  vis : str = None
  """能见度，默认单位：公里"""
  cloud : str = None
  """云量，百分比数值。可能为空"""


class QWeatherWeatherHourlyDataDto(QWeatherDto):
  """逐小时天气预报数据"""
  
  fxTime: str = None
  """预报时间"""
  temp : str = None
  """温度，默认单位：摄氏度"""
  icon : str = None
  """天气状况的图标代码"""
  text : str = None
  """天气状况的文字描述，包括阴晴雨雪等天气状态的描述"""
  wind360 : str = None
  """ 风向360角度"""
  windDir : str = None
  """风向"""
  windScale : str = None
  """风力等级"""
  windSpeed : str = None
  """风速，公里/小时"""
  humidity : str = None
  """相对湿度，百分比数值"""
  precip : str = None
  """ 当前小时累计降水量，默认单位：毫米"""
  pop : str = None
  """逐小时预报降水概率，百分比数值，可能为空"""
  pressure : str = None
  """大气压强，默认单位：百帕"""
  cloud : str = None
  """云量，百分比数值。可能为空"""
  dew : str = None
  """露点温度。可能为空"""


class QWeatherMinutelyRainDataDto(QWeatherDto):
  """分钟级降水量数据"""
  
  fxTime : str = None
  """预报时间"""
  precip : str = None
  """5分钟累计降水量，单位毫米"""
  type : str = None
  """降水类型：rain = 雨，snow = 雪"""


class QWeatherLocationDto(QWeatherDto):
  """搜索城市返回的地区详情"""
  
  id:str = None
  """地区/城市ID"""
  name:str = None
  """地区/城市名称"""
  lat:str = None
  """地区/城市纬度"""
  lon:str = None
  """地区/城市经度"""
  adm2:str = None
  """地区/城市的上级行政区划名称"""
  adm1:str = None
  """地区/城市所属一级行政区域"""
  country:str =None
  """地区/城市所属国家名称"""
  tz: str = None
  """地区/城市所在时区"""
  utcOffset:str = None
  """地区/城市目前与UTC时间偏移的小时数"""
  isDist:str = None
  """地区/城市是否当前处于夏令时。1 表示当前处于夏令时，0 表示当前不是夏令时。"""
  type:str = None
  """地区/城市的属性"""
  rank :str = None
  """地区评分"""
  fxLink : str = None
  """该地区的天气预报网页链接，便于嵌入你的网站或应用"""


class QWeatherWarningDto(QWeatherDto):
  """天气预警数据"""

  id: str = None
  """本条预警的唯一标识，可判断本条预警是否已经存在"""
  sender : str = None
  """预警发布单位，可能为空"""
  pubTime : str = None
  """预警发布时间"""
  title : str = None
  """预警信息标题"""
  startTime : str = None
  """预警开始时间，可能为空"""
  endTime : str = None
  """预警结束时间，可能为空"""
  status : str = None
  """预警信息的发布状态"""
  severity: str  = None
  """预警严重等级"""
  severityColor : str  = None
  """预警严重等级颜色，可能为空"""
  type : str = None
  """预警类型ID"""
  typeName : str = None
  """预警类型名称"""
  urgency : str = None
  """ 预警信息的紧迫程度，可能为空"""
  certainty : str = None
  """预警信息的确定性，可能为空"""
  text : str = None
  """预警详细文字描述"""
  related : str = None
  """与本条预警相关联的预警ID，当预警状态为cancel或update时返回。可能为空"""


class QWeatherWarningCityDto(QWeatherDto):
  """预警城市信息"""
  locationId : str = None
  """当前国家预警的LocationID"""
  
  
class QWeatherIndicesDailyDto(QWeatherDto):
  """天气指数每日数据"""
  date : str = None
  """预报日期"""
  type : str = None
  """ 生活指数类型ID"""
  name  : str = None
  """生活指数类型的名称"""
  level  : str = None
  """生活指数预报等级"""
  category  : str = None
  """生活指数预报级别名称"""
  text : str = None
  """生活指数预报的详细描述，可能为空"""


class QWeatherAirQualityAQIDto(QWeatherDto):
  """空气质量AQI数据"""
  code :str = None
  """空气质量指数Code"""
  name  :str = None
  """空气质量指数的名字"""
  defaultLocalAqi  :str = None
  """是否是默认/推荐的当地AQI"""
  value  :str = None
  """空气质量指数的值"""
  valueDisplay  :str = None
  """空气质量指数的值的文本显示"""
  level  :str = None
  """空气质量指数等级，可能为空"""
  category  :str = None
  """空气质量指数类别，可能为空"""
  color  :str = None
  """空气质量指数的颜色，RGB格式"""
  primaryPollutant : dict = None
  """ 首要污染物"""
  health : dict = None
  """健康指导"""
  
  
class QWeatherAirQualityPollutantDto(QWeatherDto):
  """空气质量污染物数据"""
  code: str = None
  """污染物的Code"""
  name : str = None
  """ 污染物的名字"""
  fullName : str = None
  """污染物的全称"""
  concentration: any = None
  """污染物的浓度值"""
  subIndex: any = None
  """ 污染物的分指数"""


class QWeatherAirQualityStationDto(QWeatherDto):
  """空气质量（Beta）监测站数据"""
  
  id:str = None
  """AQI相关联的监测站Location ID，可能为空"""
  name: str = None
  """AQI相关联的监测站名称"""
  
  
class QWeatherAirQualityStationPollutantDto(QWeatherDto):
  """空气质量监测站污染物数据"""
  code : str = None
  """污染物的Code"""
  name  : str = None
  """ 污染物的名字"""
  fullName  : str = None
  """污染物的全称"""
  concentration : any = None
  """污染物的浓度"""
  
  
class QWeatherAirRealDataDto(QWeatherDto):
  """空气质量实时数据"""
  pubTime : str = None
  """空气质量数据发布时间"""
  aqi  : str = None
  """空气质量指数"""
  level  : str = None
  """空气质量指数等级"""
  category  : str = None
  """空气质量指数级别"""
  pm10   : str = None
  """ PM10"""
  pm2p5  : str = None
  """ PM2.5"""
  no2  : str = None
  """二氧化氮"""
  so2  : str = None
  """ 二氧化硫"""
  co  : str = None
  """一氧化碳"""
  o3  : str = None
  """臭氧"""
  primary  : str = None
  """空气质量的主要污染物，空气质量为优时，返回值为NA"""

class QWeatherAirStationDataDto(QWeatherDto):
  """空气质量监测站数据"""
  name : str = None
  """监测站名称"""
  id  : str = None
  """监测站ID"""
  pubTime  : str = None
  """空气质量数据发布时间"""
  aqi  : str = None
  """空气质量指数"""
  level  : str = None
  """空气质量指数等级"""
  category  : str = None
  """ 空气质量指数级别"""
  primary  : str = None
  """空气质量的主要污染物，空气质量为优时，返回值为NA"""
  pm10  : str = None
  """PM10"""
  pm2p5  : str = None
  """PM2.5"""
  no2  : str = None
  """ 二氧化氮"""
  so2  : str = None
  """二氧化硫"""
  co  : str = None
  """一氧化碳"""
  o3  : str = None
  """臭氧"""
  

class QWeatherAirDailyDto(QWeatherDto):
  """空气质量每日预报数据"""
  fxDate : str = None
  """预报日期"""
  level : str = None
  """空气质量指数等级"""
  category : str = None
  """空气质量指数级别"""
  primary  : str = None
  """空气质量的主要污染物，空气质量为优时，返回值为NA"""
  aqi  : str = None
  """空气质量指数"""
  
  
class QWeatherSimpleWeatherDto(QWeatherDto):
  """每日天气简单数据"""
  date : str = None
  """当天日期"""
  sunrise  : str = None
  """当天日出时间，在高纬度地区可能为空"""
  sunset  : str = None
  """日落时间，在高纬度地区可能为空"""
  moonrise  : str = None
  """当天月升时间，可能为空"""
  moonset  : str = None
  """当天月落时间，可能为空"""
  moonPhase  : str = None
  """当天月相名称"""
  tempMax  : str = None
  """当天最高温度"""
  tempMin  : str = None
  """当天最低温度"""
  precip : str = None
  """当天总降水量，默认单位：毫米"""
  pressure  : str = None
  """大气压强，默认单位：百帕"""
  humidity  : str = None
  """当天每小时相对湿度，百分比数值"""


class QWeatherSimpleWeatherHourlyDto(QWeatherDto):
  """逐小时天气简单数据"""
  time : str = None
  """当天时间"""
  temp  : str = None
  """ 当天每小时温度，默认单位：摄氏度"""
  icon  : str = None
  """ 当天每小时天气状况的图标代码"""
  text  : str = None
  """当天每小时天气状况的文字描述，包括阴晴雨雪等天气状态的描述"""
  wind360  : str = None
  """ 当天每小时风向360角度"""
  windDir  : str = None
  """当天每小时风向"""
  windScale  : str = None
  """ 当天每小时风力等级"""
  windSpeed  : str = None
  """当天每小时风速，公里/小时"""
  humidity  : str = None
  """当天每小时相对湿度，百分比数值"""
  precip  : str = None
  """当天每小时累计降水量，默认单位：毫米"""
  pressure  : str = None
  """大气压强，默认单位：百帕"""


class QWeatherSimpleAirHourlyDto(QWeatherDto):
  """逐小时空气质量简单数据"""
  pubTime : str = None
  """空气质量数据发布时间"""
  aqi  : str = None
  """空气质量指数"""
  level  : str = None
  """空气质量指数等级"""
  category  : str = None
  """空气质量指数级别"""
  primary  : str = None
  """空气质量的主要污染物，空气质量为优时，返回值为NA"""
  pm10  : str = None
  """PM10"""
  pm2p5  : str = None
  """ PM2.5"""
  no2  : str = None
  """二氧化氮"""
  so2  : str = None
  """二氧化硫"""
  co  : str = None
  """一氧化碳"""
  o3  : str = None
  """臭氧"""


class QWeatherTropicalCycloneForecastDto(QWeatherDto):
  """台风预报数据"""
  fxTime : str = None
  """台风预报时间"""
  lat  : str = None
  """台风所处纬度"""
  lon  : str = None
  """台风所处经度"""
  type  : str = None
  """ 台风类型"""
  pressure  : str = None
  """台风中心气压"""
  windSpeed : str = None
  """台风附近最大风速"""
  moveSpeed  : str = None
  """台风移动速度"""
  moveDir  : str = None
  """台风移动方位"""
  move360  : str = None
  """台风移动方位360度方向"""
  

class QWeatherTropicalCycloneDataDto(QWeatherDto):
  """台风实况数据"""
  pubTime : str = None
  """台风预报时间"""
  lat  : str = None
  """台风所处纬度"""
  lon  : str = None
  """台风所处经度"""
  type  : str = None
  """ 台风类型"""
  pressure  : str = None
  """台风中心气压"""
  windSpeed : str = None
  """台风附近最大风速"""
  moveSpeed  : str = None
  """台风移动速度"""
  moveDir  : str = None
  """台风移动方位"""
  move360  : str = None
  """台风移动方位360度方向"""  
  windRadius30: any  = None
  """当前台风7级风圈信息，可能为空"""
  windRadius50: any = None
  """台风10级风圈信息，可能为空"""
  windRadius64: any = None
  """台风12级风圈信息，可能为空"""
  
  
class   QWeatherTropicalCycloneStormDto(QWeatherDto):
  """台风信息"""
  id : str = None
  """台风ID"""
  stormId  : str = None
  """台风ID"""
  name  : str = None
  """台风名称"""
  basin  : str = None
  """台风所处流域"""
  year  : str = None
  """ 台风所处年份"""
  isActive  : str = None
  """是否为活跃台风。1 活跃台风，0 停编。"""


class QWeatherOceanTideDto(QWeatherDto):
  """潮汐数据"""
  fxTime : str = None
  height  : str = None
  type  : str = None
  
  
class QWeatherOceanTideHourlyDto(QWeatherDto):
  """潮汐逐小时数据"""
  fxTime : str = None
  height  : str = None
  

class QWeatherOceanCurrentsDto(QWeatherDto):
  """潮流数据"""
  fxTime : str = None
  """潮流最大流速时间"""
  speedMax   : str = None
  """ 潮流最大流速，单位：厘米/秒"""
  dir360   : str = None
  """潮流360度方向"""
  
  
class QWeatherOceanCurrentsHourlyDto(QWeatherDto):
  """潮流逐小时数据"""
  fxTime : str = None
  """潮流逐小时预报时间"""
  speed    : str = None
  """ 潮流流速，单位：厘米/秒"""
  dir360   : str = None
  """潮流360度方向"""
  

class QWeatherSolarRadiationDto(QWeatherDto):
  """太阳辐射数据"""
  fxTime : str = None
  """逐小时预报时间"""
  net : str = None
  """净太阳辐射，W/m2"""
  diffuse : str = None
  """太阳散射辐射，W/m2"""
  direct : str = None
  """太阳直接辐射，W/m2"""
  

class QWeatherMoonPhaseDto(QWeatherDto):
  """月相数据"""
  fxTime : str = None
  """月相逐小时预报时间"""
  value : str = None
  """月相数值"""
  name : str = None
  """月相名称"""
  icon : str = None
  """月相图标代码"""
  illumination : str = None
  """月亮照明度，百分比数值"""

class INDICES_TYPE(Enum):
  """天气指数类型"""
  
  ALL     = '0'
  """全部天气指数"""
  
  SPT     = '1'
  """运动指数"""
  
  CW      = '2'
  """洗车指数"""
  
  DRSG    = '3'
  """穿衣指数"""
  
  FIS     = '4'
  """钓鱼指数"""
  
  UV      = '5'
  """紫外线指数"""
  
  TRA     = '6'
  """旅游指数"""
  
  AG      = '7'
  """花粉过敏指数"""
  
  COMF    = '8'
  """舒适度指数"""
  
  FLU     = '9'
  """感冒指数"""
  
  AP      = '10'
  """空气污染扩散条件指数"""
  
  AC      = '11'
  """空调开启指数"""
  
  GL      = '12'
  """太阳镜指数"""
  
  MU      = '13'
  """化妆指数"""
  
  DC      = '14'
  """晾晒指数"""
  
  PTFC    = '15'
  """交通指数"""
  
  SPI     = '16'
  """防晒指数"""



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherAirRealDataDto, QWeatherAirStationDataDto


class AirNowRequest(QWeatherRequestBase):
  
  _PATH = '/air/now'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.location = self.get_arg('location', kwargs, required=True)
    self.lang = self.get_arg('lang', kwargs)
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class AirNowResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.now:QWeatherAirRealDataDto = self.get_obj('now', kwargs, QWeatherAirRealDataDto)
    self.station:list[QWeatherAirStationDataDto] = self.get_items('station', kwargs, QWeatherAirStationDataDto)
    


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.solar_radiation.solar_radiation_hourly import SolarRadiationHourlyRequest, SolarRadiationHourlyResponse


class QWeatherSolarRadiationPack(QWeatherPackBase):
  """_summary_

  Args:
      QWeatherPackBase (_type_): _description_
  """
  
  def hourly_24h(self, location:str)  -> SolarRadiationHourlyResponse:
    """太阳辐射24小时预报

    Args:
        location (str): (必选)需要查询地区的以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位）。例如 location=116.41,39.92

    Returns:
        SolarRadiationHourlyResponse: _description_
    """
    req = SolarRadiationHourlyRequest('24h', location=location)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return SolarRadiationHourlyResponse(**data)
  
  
  def hourly_72h(self, location:str)  -> SolarRadiationHourlyResponse:
    """太阳辐射72小时预报

    Args:
        location (str): (必选)需要查询地区的以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位）。例如 location=116.41,39.92

    Returns:
        SolarRadiationHourlyResponse: _description_
    """
    req = SolarRadiationHourlyRequest('72h', location=location)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return SolarRadiationHourlyResponse(**data)



from pyqweather import QWeatherRequestBase, QWeatherResponseBase

class AstronomySolarElevationAngleRequest(QWeatherRequestBase):
  
  _PATH = '/astronomy/solar-elevation-angle'
  
  def __init__(self, **kwargs):
    
    super().__init__(**kwargs)
    self.location       = self.get_arg('location', kwargs, required=True)
    self.date           = self.get_arg('date', kwargs, required=True)
    self.time           = self.get_arg('time', kwargs, required=True)
    self.tz             = self.get_arg('tz', kwargs, required=True)
    self.alt            = self.get_arg('alt', kwargs, required=True)
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class AstronomySolarElevationAngleResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime               = self.get_arg('updateTime', kwargs, None)
    self.solarElevationAngle      = self.get_arg('solarElevationAngle', kwargs, None)
    self.solarAzimuthAngle        = self.get_arg('solarAzimuthAngle', kwargs, None)
    self.solarHour                = self.get_arg('solarHour', kwargs, None)
    self.hourAngle                = self.get_arg('hourAngle', kwargs, None)
    



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherLocationDto


class CityTopRequest(QWeatherRequestBase):
  
  _PATH = '/city/top'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.range = self.get_arg('range', kwargs)
    self.number = self.get_arg('number', kwargs)
    self.lang = self.get_arg('lang', kwargs)
    
    
  def __str__(self) -> str:
    return f'location={self.range}, number={self.number}, lang={self.lang}'
  


class CityTopResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.top_city_list:list[QWeatherLocationDto] = self.get_items('topCityList', kwargs, QWeatherLocationDto)
    



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherWarningCityDto


class WarningListRequest(QWeatherRequestBase):
  """获取指定国家或地区当前正在发生天气灾害预警的城市列表"""
  
  _PATH = '/warning/list'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.range = self.get_arg('range', kwargs, required=True)
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  


class WarningListResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.warningLocList:list[QWeatherWarningCityDto] = self.get_items('warningLocList', kwargs, QWeatherWarningCityDto)
    



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherOceanCurrentsDto, QWeatherOceanCurrentsHourlyDto


class CurrentsRequest(QWeatherRequestBase):
  """未来10天全球潮流数据，包括潮流流速和流向。
  """
  
  _PATH = '/ocean/currents'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.location = self.get_arg('location', kwargs, required=True)
    self.date = self.get_arg('date', kwargs, required=True)
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class CurrentsResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.currentsTable:list[QWeatherOceanCurrentsDto] = self.get_items('currentsTable', kwargs, QWeatherOceanCurrentsDto)
    self.currentsHourly:list[QWeatherOceanCurrentsHourlyDto] = self.get_items('currentsHourly', kwargs, QWeatherOceanCurrentsHourlyDto)
    


#coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.air.air_now import AirNowRequest, AirNowResponse
from pyqweather.packages.air.air_daily import AirDailyRequest, AirDailyResponse


class QWeatherAirPack(QWeatherPackBase):
  """空气质量
  """
  
  def air_now(self, location, lang:str='zh-hans'):
    """实时空气质量"""
    
    req = AirNowRequest(location=location,lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return AirNowResponse(**data)
  
  
  def air_5d(self, location:str, lang:str='zh-hans'):
    """空气质量每日预报

    Args:
        location (str): (必选)需要查询地区的LocationID或以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位），LocationID可通过GeoAPI获取。例如 location=101010100 或 location=116.41,39.92
        lang (str, optional): 多语言设置，请阅读多语言文档，了解我们的多语言是如何工作、如何设置以及数据是否支持多语言。 Defaults to 'zh-hans'.

    Returns:
        _type_: _description_
    """
    
    req = AirDailyRequest(day_range='5d', location=location,lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return AirDailyResponse(**data)


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.weather_warning.warning_now import WarningNowRequest, WarningNowResponse
from pyqweather.packages.weather_warning.warning_list import WarningListRequest, WarningListResponse

class QWeatherWeatherWarningPack(QWeatherPackBase):
  """天气灾害预警API包"""
  
  def warning_now(self, location, lang:str='zh-hans'):
    """天气灾害预警

    Args:
        location (_type_): (必选)需要查询地区的LocationID或以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位），LocationID可通过GeoAPI获取。例如 location=101010100 或 location=116.41,39.92
        lang (str, optional):多语言设置，请阅读多语言文档，了解我们的多语言是如何工作、如何设置以及数据是否支持多语言。 Defaults to 'zh-hans'.

    Returns:
        _type_: _description_
    """    
    
    req = WarningNowRequest(location=location,lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WarningNowResponse(**data)
  
  
  def warning_list(self, range: str):
    """获取指定国家或地区当前正在发生天气灾害预警的城市列表

    Args:
        range (str): 选择指定的国家或地区，使用ISO 3166格式。
        例如range=cn或range=hk。目前该功能仅支持中国（包括港澳台）地区的城市列表，其他国家和地区请使用请使用[天气灾害预警]单独获取。
    """   
    req = WarningListRequest(range=range)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WarningListResponse(**data) 
    
  


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.ocean.tide import TideRequest, TideResponse
from pyqweather.packages.ocean.currents import CurrentsRequest, CurrentsResponse


class QWeatherOceanPack(QWeatherPackBase):
  
  
  def tide(self, location:str, date:str):
    """未来10天全球潮汐数据，包括满潮、干潮高度和时间，逐小时潮汐数据。

    Args:
        location (str): (必选)需要查询的潮汐站点，请填写潮汐站点的LocationID，LocationID可通过POI搜索服务获取。例如 location=P2951
        date (str): (必选)选择日期，最多可选择未来10天（包含今天）的数据。日期格式为yyyyMMdd，例如 date=20200531

    Returns:
        TideResponse: _description_
    """
    req = TideRequest(location=location, date=date)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return TideResponse(**data)
  
  
  def currents(self, location:str, date:str):
    """未来10天全球潮流数据，包括潮流流速和流向。

    Args:
        location (str): (必选)需要查询的潮汐站点，请填写潮汐站点的LocationID，LocationID可通过POI搜索服务获取。例如 location=P2951
        date (str): (必选)选择日期，最多可选择未来10天（包含今天）的数据。日期格式为yyyyMMdd，例如 date=20200531

    Returns:
        CurrentsResponse: _description_
    """
    req = CurrentsRequest(location=location, date=date)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return CurrentsResponse(**data)



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherTropicalCycloneDataDto


class StormTrackRequest(QWeatherRequestBase):
  
  _PATH = '/tropical/storm-track'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.stormid = self.get_arg('stormid', kwargs, required=True)
    
    
  def __str__(self) -> str:
    return f'stormid={self.stormid}'
  

class StormTrackResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.isActive = self.get_arg('isActive', kwargs, None)
    """是否为活跃台风。1 活跃台风，0 停编。"""
    self.now:QWeatherTropicalCycloneDataDto = self.get_obj('now', kwargs, QWeatherTropicalCycloneDataDto)
    self.track:list[QWeatherTropicalCycloneDataDto] = self.get_items('track', kwargs, QWeatherTropicalCycloneDataDto)
    



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherOceanTideDto, QWeatherOceanTideHourlyDto


class TideRequest(QWeatherRequestBase):
  """未来10天全球潮汐数据，包括满潮、干潮高度和时间，逐小时潮汐数据。"""
  
  _PATH = '/ocean/tide'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.location = self.get_arg('location', kwargs, required=True)
    self.date = self.get_arg('date', kwargs, required=True)
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class TideResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.tideTable:list[QWeatherOceanTideDto] = self.get_items('tideTable', kwargs, QWeatherOceanTideDto)
    self.tideHourly:list[QWeatherOceanTideHourlyDto] = self.get_items('tideHourly', kwargs, QWeatherOceanTideHourlyDto)
    



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherMoonPhaseDto


class AstronomyMoonRequest(QWeatherRequestBase):
  
  _PATH = '/astronomy/moon'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.location = self.get_arg('location', kwargs, required=True)
    self.date = self.get_arg('date', kwargs, required=True)
    self.lang = self.get_arg('lang', kwargs)
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class AstronomyMoonResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.moonrise= self.get_arg('sunrise', kwargs, None)
    self.moonset= self.get_arg('sunset', kwargs, None)
    self.moonPhase:list[QWeatherMoonPhaseDto]= self.get_items('moonPhase', kwargs, QWeatherMoonPhaseDto)



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherAirQualityAQIDto, QWeatherAirQualityPollutantDto, QWeatherAirQualityStationDto


class AirQualityNowRequest(QWeatherRequestBase):
  
  _PATH = '/now'
  
  def __init__(self, locationId:str,  **kwargs):
    super().__init__(**kwargs)
    self.locationId = locationId
    # self.pollutant = self.get_arg('pollutant', kwargs, False)
    # self.station = self.get_arg('station', kwargs, True)
    self.lang = self.get_arg('lang', kwargs)
    
    self._PATH = f'/now/{self.locationId}'
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  


class AirQualityNowResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.aqi:list[QWeatherAirQualityAQIDto] = self.get_items('aqi', kwargs, QWeatherAirQualityAQIDto)
    self.pollutant:list[QWeatherAirQualityPollutantDto] = self.get_items('pollutant', kwargs, QWeatherAirQualityPollutantDto)
    self.station:list[QWeatherAirQualityStationDto] = self.get_items('station', kwargs, QWeatherAirQualityStationDto)
    
    



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherSolarRadiationDto


class SolarRadiationHourlyRequest(QWeatherRequestBase):
  """太阳辐射逐小时预报。
  """
  
  _PATH = '/solar-radiation'
  
  def __init__(self,hour_range:str,  **kwargs):
    """构建太阳辐射逐小时预报的请求类

    Args:
        hour_range (str): ONLY  24h  72h
        **kwargs (dict): 
        
          - location: (必选)需要查询地区的以英文逗号分隔的经度,纬度坐标（十进制，最多支持小数点后两位）。例如 location=116.41,39.92
    """
    super().__init__(**kwargs)
    self.location = self.get_arg('location', kwargs, required=True)
    
    self._PATH = f'/solar-radiation/{hour_range}'
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class SolarRadiationHourlyResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.radiation:list[QWeatherSolarRadiationDto] = self.get_items('radiation', kwargs, QWeatherSolarRadiationDto)
    """除非特别说明，本数据返回的太阳辐射均指地表垂直向下的短波辐射，单位w/m2"""
    


#coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.air_quality.airquality_now import AirQualityNowRequest, AirQualityNowResponse
from pyqweather.packages.air_quality.airquality_station import AirQualityStationRequest, AirQualityStationResponse

class QWeatherAirQualityPack(QWeatherPackBase):
  """全球空气质量"""
  
  
  def airquality_now(self, locationId:str, lang:str='zh-hans'):
    """实时空气质量"""
    
    req = AirQualityNowRequest(locationId=locationId,  lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return AirQualityNowResponse(**data)

  def airquality_station(self, locationId:str, lang:str='zh-hans'):
    """监测站数据"""
    
    req = AirQualityStationRequest(locationId=locationId, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return AirQualityStationResponse(**data)


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.weather.weather_now import WeatherNowRequest, WeatherNowResponse
from pyqweather.packages.weather.weather_daily import WeatherDailyRequest, WeatherDailyResponse
from pyqweather.packages.weather.weather_hourly import WeatherHourlyRequest, WeatherHourlyResponse



class QWeatherWeatherPack(QWeatherPackBase):
  
 
  def weather_now(self, location, lang:str='zh-hans', unit:str='m'):
    """实时天气"""
    req = WeatherNowRequest(location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherNowResponse(**data)
  
  
  def weather_3d(self, location, lang:str='zh-hans', unit:str='m'):
    """3日天气"""
    req = WeatherDailyRequest(day_range='3d', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherDailyResponse(**data)
  
  
  def weather_7d(self, location, lang:str='zh-hans', unit:str='m'):
    """7日天气"""
    req = WeatherDailyRequest(day_range='7d', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherDailyResponse(**data)
  
  
  def weather_10d(self, location, lang:str='zh-hans', unit:str='m'):
    """10日天气"""
    req = WeatherDailyRequest(day_range='10d', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherDailyResponse(**data)
  
  
  def weather_10d(self, location, lang:str='zh-hans', unit:str='m'):
    """15日天气"""
    req = WeatherDailyRequest(day_range='15d', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherDailyResponse(**data)
  
  
  def weather_30d(self, location, lang:str='zh-hans', unit:str='m'):
    """30日天气"""
    req = WeatherDailyRequest(day_range='30d', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherDailyResponse(**data)
  
  
  def weather_24h(self, location, lang:str='zh-hans', unit:str='m'):
    req = WeatherHourlyRequest(hour_range='24h', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherHourlyResponse(**data)
  
  
  def weather_72h(self, location, lang:str='zh-hans', unit:str='m'):
    req = WeatherHourlyRequest(hour_range='72h', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherHourlyResponse(**data)
  
  
  def weather_168h(self, location, lang:str='zh-hans', unit:str='m'):
    req = WeatherHourlyRequest(hour_range='168h', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return WeatherHourlyResponse(**data)



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherLocationDto


class PoiLookupRequest(QWeatherRequestBase):
  
  _PATH = '/poi/lookup'
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.location = self.get_arg('location', kwargs, required=True)
    self.type = self.get_arg('type', kwargs, required=True)
    self.city = self.get_arg('city', kwargs)
    self.number = self.get_arg('number', kwargs)
    self.lang = self.get_arg('lang', kwargs)
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  


class PoiLookupResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.poi:list[QWeatherLocationDto] = self.get_items('poi', kwargs, QWeatherLocationDto)
    


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.tropical_cyclone.storm_forecast import StormForecastRequest, StormForecastResponse
from pyqweather.packages.tropical_cyclone.storm_track import StormTrackRequest, StormTrackResponse
from pyqweather.packages.tropical_cyclone.storm_list import StormListRequest, StormListResponse


class QWeatherTropicalCyclonePack(QWeatherPackBase):
  """热带气旋（台风）API"""
  
  def storm_forecast(self, stormid:str) -> StormForecastResponse:
    """台风预报API提供全球主要海洋流域的台风预测位置、等级、气压、风速等。
    如果查询的台风已经结束，则返回的数据为空，建议先通过台风列表接口获取台风的状态
    
    Args:
        stormid (str): (必选)需要查询的台风ID，StormID可通过台风查询API获取。例如 stormid=NP2018

    Returns:
        HistoricalWeatherResponse: _description_
    """
    
    req = StormForecastRequest(stormid=stormid)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return StormForecastResponse(**data)
  
  
  def storm_track(self, stormid:str) -> StormTrackResponse:
    """台风实况和路径。
    台风实况和路径API提供全球主要海洋流域的台风实时位置、等级、气压、风速以及活跃台风的轨迹路径。

    Args:
        stormid (str): (必选)需要查询的台风ID，StormID可通过台风查询API获取。例如 stormid=NP2018

    Returns:
        StormTrackResponse: _description_
    """
    
    req = StormTrackRequest(stormid=stormid)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return StormTrackResponse(**data)
  
  
  def storm_list(self, year:str, basin:str='NP') -> StormListResponse:
    """台风列表API提供全球主要海洋流域最近2年的台风列表。```目前仅支持中国沿海地区，即basin=NP```

    Args:
        year (str): (必选)支持查询本年度和上一年度的台风，例如：year=2020, year=2019
        
        basin (str, optional): (必选)需要查询的台风所在的流域，例如中国处于西北太平洋，即 basin=NP。当前仅支持NP。Defaults to 'NP'.
    """
    
    req = StormListRequest(year=year, basin=basin)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return StormListResponse(**data)



from pyqweather import QWeatherRequestBase, QWeatherResponseBase
from pyqweather.packages import QWeatherWeatherDailyDataDto


class GridWeatherDailyRequest(QWeatherRequestBase):
  
  _PATH = '/grid-weather'
  
  def __init__(self, day_range: str, **kwargs):
    super().__init__(**kwargs)
    self._PATH = f'/grid-weather/{day_range}'
    self.location = self.get_arg('location', kwargs, required=True)
    self.lang = self.get_arg('lang', kwargs)
    self.unit = self.get_arg('unit', kwargs)
    
    
  def __str__(self) -> str:
    return f'location={self.location}'
  

class GridWeatherDailyResponse(QWeatherResponseBase):
  
  def __init__(self, **kwargs):
    super().__init__(**kwargs)
    self.updateTime = self.get_arg('updateTime', kwargs, None)
    self.fxLink = self.get_arg('fxLink', kwargs, None)
    self.daily:list[QWeatherWeatherDailyDataDto] = self.get_items('daily', kwargs, QWeatherWeatherDailyDataDto)
    


# coding: utf-8

from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='pyqweather',
    version='1.0.3',
    description='和风天气 api for python3', 
    author='xuwh',  
    author_email='xuwhdev@gmail.com',
    url='https://github.com/imaxu/py-qweather',
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6",
)



from abc import abstractmethod, ABC
import hashlib
import os
import time


class AuthCredentialBase(ABC):

  def __init__(self, **kwargs):
    self._kwargs = kwargs

  @abstractmethod
  def is_valid(self) -> bool:
    return True

  @abstractmethod
  def create_sign_param(self, **kwargs) -> dict[str, str]:
    return {}
  


class SimpleAuthCredential(AuthCredentialBase):
  """简单认证凭据， 直接使用ApiKey

  Args:
      AuthCredentialBase (_type_): _description_
  """

  def __init__(self, app_key: str):
    super().__init__(app_key=app_key)
    
    self.app_key = app_key

  def is_valid(self):
    return super().is_valid() and self.app_key is not None and len(self.app_key) > 0

  def create_sign_param(self, **kwargs) -> dict[str, str]:
    return {'key': self.app_key }


class EnvironmentVariableSimpleAuthCredential(SimpleAuthCredential):
  
  def __init__(self):
    super().__init__(os.getenv('QWEATHER_SDK_APP_KEY', ''))



class SignAuthCredential(AuthCredentialBase):
  """数字签名认证凭据

  Args:
      AuthCredentialBase (_type_): _description_
  """

  def __init__(self, public_id: str, app_key: str):
    super().__init__(public_id=public_id, app_key=app_key)
    self.app_key = app_key
    self.public_id = public_id
    self.t = int(time.time())

  def _generate_signature(self, parameters: dict[str, str | int]):
    """
    生成签名的函数。
    :param parameters: 包含请求参数的字兂数组
    :param api_key: 用于签名的API密钥
    :return: 生成的签名
    """
    
    parameters.update({'t': self.t, 'publicid': self.public_id})
    
    # 1. 将请求参数格式化为“key=value”格式
    key_value_pairs = [f"{key}={value}" for key, value in parameters.items() if value]
    
    # 2. 去除请求参数中值为空的参数（已在上一步完成）
    # 3. 去除请求参数中参数为'sign'的参数
    key_value_pairs = [item for item in key_value_pairs if not item.startswith('sign=')]
    
    # 4. 将剩余的键值对以字典序升序排列，并用'&'符号连接
    key_value_pairs.sort()
    sorted_str = '&'.join(key_value_pairs)
    # 5. 将排好序的字符串后拼接上API密钥
    string_to_sign = f'{sorted_str}{self.app_key}'
    # 6. 将得到的字符串进行MD5加密
    md5_hash = hashlib.md5()
    #print(string_to_sign)
    md5_hash.update(string_to_sign.encode('utf-8'))
    signature = md5_hash.hexdigest()
    #print(signature)
    
    return signature
  
    
  def is_valid(self):
    return super().is_valid() \
      and self.app_key is not None \
        and len(self.app_key) > 0 \
          and self.public_id is not None \
            and len(self.public_id) > 0
            
            
  def create_sign_param(self, **kwargs):
    return { 'sign' : self._generate_signature(kwargs), 't': self.t, 'publicid': self.public_id }  
  
  
class EnvironmentVariableSignAuthCredential(SignAuthCredential):
  
  def __init__(self):
    super().__init__(os.getenv('QWEATHER_SDK_PUBLIC_ID', ''), os.getenv('QWEATHER_SDK_APP_KEY', ''))


#coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.geo.city_lookup import CityLookupRequest, CityLookupResponse
from pyqweather.packages.geo.city_top import CityTopRequest, CityTopResponse
from pyqweather.packages.geo.poi_lookup import PoiLookupRequest, PoiLookupResponse
from pyqweather.packages.geo.poi_range import PoiRangeRequest, PoiRangeResponse
import inspect

class QWeatherGeoPack(QWeatherPackBase):
     
    
  def city_lookup(self, location:str, adm:str=None, range:str='cn', number:int=10, lang:str='zh-hans'):
    """城市搜索"""
    req = CityLookupRequest(location=location, adm=adm, range=range, number=number, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return CityLookupResponse(**data)
  
  
  def city_top(self, range:str='cn', number:int=10, lang:str='zh-hans'):
    """热门城市列表"""
    req = CityTopRequest(range=range, number=number, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return CityTopResponse(**data)
  
  
  def poi_lookup(self, location:str, type:str, city:str=None, number:int=10, lang:str='zh-hans'):
    """POI搜索"""
    
    req = PoiLookupRequest(location=location,type=type, city=city,  number=number, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return PoiLookupResponse(**data)  
  
  
  def poi_range(self, location:str, type:str, radius:str=None, number:int=10, lang:str='zh-hans'):
    """POI范围搜索"""
      
    req = PoiRangeRequest(location=location,type=type, radius=radius,  number=number, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return PoiRangeResponse(**data)  


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.time_machine.historical_weather import HistoricalWeatherRequest, HistoricalWeatherResponse
from pyqweather.packages.time_machine.historical_air import HistoricalAirRequest, HistoricalAirResponse

class QWeatherTimeMachinePack(QWeatherPackBase):
  """时光机API"""
  
  def historical_weather(self, location:str, date:str, unit:str='m', lang:str='zh-hans'):
    """获取最近10天某日历史再分析数据。"""
    
    req = HistoricalWeatherRequest(location=location, date=date, unit=unit, lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return HistoricalWeatherResponse(**data)
  
  
  def historical_air(self, location:str, date:str, lang:str='zh-hans'):
    """获取最近10天某日的中国空气质量历史再分析数据。"""
    
    req = HistoricalAirRequest(location=location, date=date,  lang=lang)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return HistoricalAirResponse(**data)


# coding: utf-8
from pyqweather import QWeatherPackBase
from pyqweather.packages.grid_weather.grid_weather_now import GridWeatherNowRequest, GridWeatherNowResponse
from pyqweather.packages.grid_weather.grid_weather_daily import GridWeatherDailyRequest, GridWeatherDailyResponse
from pyqweather.packages.grid_weather.grid_weather_hourly import GridWeatherHourlyRequest, GridWeatherHourlyResponse


class QWeatherGridWeatherPack(QWeatherPackBase):
  """格点天气API包"""
  
  def grid_weather_now(self, location, lang:str='zh-hans', unit:str='m'):
    """格点实况天气"""
    
    req = GridWeatherNowRequest(location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return GridWeatherNowResponse(**data)
  
  
  def grid_weather_24h(self, location, lang:str='zh-hans', unit:str='m'):
    """逐小时预报（未来24小时）"""
    
    req = GridWeatherHourlyRequest(hour_range='24h', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return GridWeatherHourlyResponse(**data)
  
  
  def grid_weather_72h(self, location, lang:str='zh-hans', unit:str='m'):
    """逐小时预报（未来72小时）"""
    
    req = GridWeatherHourlyRequest(hour_range='72h', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return GridWeatherHourlyResponse(**data)
  
  
  def grid_weather_3d(self, location, lang:str='zh-hans', unit:str='m'):
    """格点3天预报"""
    
    req = GridWeatherDailyRequest(day_range='3d', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return GridWeatherDailyResponse(**data)
  
  
  def grid_weather_7d(self, location, lang:str='zh-hans', unit:str='m'):
    """格点7天预报"""
    
    req = GridWeatherDailyRequest(day_range='7d', location=location,lang=lang, unit=unit)
    data = req.with_url(self.get_conf().endpoint).with_credential(self.get_conf().get_credential()).get()
    return GridWeatherDailyResponse(**data)
  
  